"""Second seat card play."""

from typing import List, Union, Tuple
import inspect

from bridgeobjects import SUITS, Card, Denomination, Suit
from bfgsupport import Board, Trick
from ..logger import log

import bfgcardplay.source.global_variables as global_vars
from .player import Player


class SecondSeat():
    def __init__(self, player: Player):
        self.player = player

    def _select_card_if_void(self, player: Player, trick: Trick) -> Card:
        """Return card if cannot follow suit."""
        player.record_void(trick.suit)
        manager = global_vars.manager
        # Trump if appropriate
        if player.trump_suit:
            pass

        suit = self._best_suit(player)

        # Signal suit preference."""
        if not manager.signal_card[player.seat]:
            suit_cards = player.suit_cards[suit.name]
            for card in suit_cards:
                if not card.is_honour:
                    manager.signal_card[player.seat] = card
                    log(inspect.stack(), f'{card}')
                    return card

            other_suit = player.other_suit_for_signals(suit)
            other_suit_cards = player.suit_cards[other_suit]
            if other_suit_cards:
                manager.signal_card[player.seat] = other_suit_cards[-1]
                log(inspect.stack(), f'{other_suit_cards[-1]}')
                return other_suit_cards[-1]

            for suit_name in SUITS:
                if suit_name != suit.name and suit_name != other_suit:
                    final_suit_cards = player.suit_cards[suit_name]
                    if final_suit_cards:
                        log(inspect.stack(), f'{final_suit_cards[-1]}')
                        return final_suit_cards[-1]

        candidate_suits = []
        for suit_name in SUITS:
            if player.unplayed_cards[suit]:
                card_count = len(player.unplayed_cards[suit])
                if card_count > len(player.total_unplayed_cards[suit]) - card_count:
                    log(inspect.stack(), f'{player.unplayed_cards[suit][-1]}')
                    return player.unplayed_cards[suit][-1]


        log(inspect.stack(), f'{player.suit_cards[suit.name][0]}')
        return player.suit_cards[suit.name][0]

    def _best_suit(self, player: Player) -> Suit:
        """Select suit for signal."""
        # TODO handle no points and equal suits
        cards = player.hand_cards.list
        suit_points = player.get_suit_strength(cards)
        max_points = 0
        best_suit = None
        for suit in SUITS:
            hcp = suit_points[suit]
            if hcp > max_points:
                max_points = hcp
                best_suit = suit
        if not best_suit:
            return player.longest_suit
        return Suit(best_suit)
